/* 
   Cracking Engine,

     LoadCharSet()
     LoadLenSet()

   Copyright (C) Bernardo Reino (reinob@jazzfree.com)
   20021120

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

/*
 * K_CHARSET = Vector of characters included in brute-force keyspace
 * K_CHARSET_LEN = Length of vector
 *
 * K_LENSET = Vector of password-lengths to try
 * K_LENSET_LEN = Length of vector
 */

#include <stdio.h>                /* [f]printf() */
#include <string.h>               /* memset() */
#include <stdlib.h>               /* strtoul() */

#include "global.h"

int LoadCharSet(char *set) {
  int cmap[256], stored = 0, next;
  enum { SEEK, NEXT, CSET } state;

  memset(cmap, 0, sizeof(cmap));

  for(state = SEEK; (next = *set) != '\0'; set ++) {
    switch(state) {
    case SEEK:
      stored = next;
      state = NEXT;
      break;

    case NEXT:
      if(next == '-') {
	state = CSET;
      } else {
	cmap[stored] ++;
	stored = next;
      }
      break;

    case CSET:
      if(next == '-') { /* double '-', weird.. */
	fprintf(stderr, 
		"alert: specified character set might not be "
		"what you are thinking of! :)\n");
	cmap[stored] ++;
	stored = next;
      } else {
	int j;

	for(j = stored; j <= next; j ++)
	  cmap[j] ++;

	state = SEEK;
      }
      break;
    }
  }

  switch(state) {
  case CSET:
    fprintf(stderr, "failed: character set not closed (%c-?)\n", stored);
    return -1;

  case NEXT:
    cmap[stored] ++; /* don't forget me! */
    break;

  case SEEK:
    break;
  }

  for(stored = next = 0; next < 256; next ++) {
    if(cmap[next] > 1) {
      fprintf(stderr, 
	      "alert: character '%c' repeated %d times in set, fixing..\n",
	      next, cmap[next]);
    }

    if(cmap[next] > 0)
      K_CHARSET[stored ++] = next;
  }

  K_CHARSET_LEN = stored;

  if(verbose) {
    fprintf(stderr, "loaded: CSET[%d] = { ", K_CHARSET_LEN);

    if(stored > 50) {
      int midpoint = (stored+1) >> 1;

      fprintf(stderr, "\n");
      
      for(next = 0; next < stored; next ++) {
	if((next % midpoint) == 0)
	  fprintf(stderr, "  ");
      
	fputc(K_CHARSET[next], stderr);
      
	if(next && ((next+1) % midpoint) == 0)
	  fprintf(stderr, "\n");
      }

      if(next % midpoint)
	fprintf(stderr, "\n");

      fprintf(stderr, "}\n");
    } else {
      for(next = 0; next < stored; next ++)
	fputc(K_CHARSET[next], stderr);
      fprintf(stderr, " }\n");
    }
  }

  return 0;
}

int LoadLenSet(char *set) {
  int lmap[16];
  int stored = 0, next;
  int in_set = 0, eol = 0;

  memset(lmap, 0, sizeof(lmap));

  while((! eol) && *set) {
    char *endptr;
    size_t seg_len;

    seg_len = strspn(set, "x0123456789");
    if(seg_len) {
      next = strtoul(set, &endptr, 0);

      if(endptr != (set + seg_len))
	goto bad_format;

      if((next < 1) || (next > MAX_PASS_LEN)) {
	fprintf(stderr, "failed: size of password must be < %d\n", 
		MAX_PASS_LEN);
	return -1;
      }

      set = endptr;

      switch(*set) {
      case '\0':
      case ',':
	if(in_set) {
	  int j;

	  for(j = stored; j <= next; j ++)
	    lmap[j - 1] ++;

	  in_set = 0;
	} else {
	  lmap[next - 1] ++;
	}

	if(! *set)
	  eol = 1;
	else
	  set ++;
	break;

      case '-':
	if(in_set) 
	  goto bad_format;

	stored = next;
	in_set = 1;
	set ++;
	break;

      default:
	fprintf(stderr, 
		"failed: invalid character ('%c') found in length-set\n", 
		*set);
	return -1;
      }
    } else {
    bad_format:
      fprintf(stderr, "failed: bad length-set format\n");
      return -1;
    }
  }

  for(stored = next = 0; next < 16; next ++) {
    if(lmap[next] > 1) {
      fprintf(stderr, "alert: length(%d) specified %u times\n",
	      next, lmap[next]);
    }

    if(lmap[next])
      K_LENSET[stored ++] = next + 1;
  }

  K_LENSET_LEN = stored;

  if(verbose) {
    fprintf(stderr, "loaded: LSET[%d] = { ", K_LENSET_LEN);

    for(next = 0; next < stored; next ++) {
      fprintf(stderr, "%d ", K_LENSET[next]);
    }
    fprintf(stderr, "}\n");
  }

  return 0;
}
